package core.api.controller;

import core.api.models.account.AccountDeleteRequest;
import core.api.models.account.EmailChangeRequest;
import core.api.models.account.PasswordChangeRequest;
import core.api.models.auth.SimpleStatusResponse;
import core.api.models.users.UserInfo;
import core.services.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.util.Optional;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api")
public class AccountController {

    @Autowired
    private UserService userService;

    @GetMapping("/account/user-details")
    public ResponseEntity<?> getUserDetails() {
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        Optional<UserInfo> userInfo = userService.getUserByName(username);
        if(userInfo.isPresent()) {
            return ResponseEntity.ok(userInfo.get());
        } else {
            return ResponseEntity.status(409).body(new SimpleStatusResponse("failed"));
        }
    }

    @PostMapping("/account/change-email")
    public ResponseEntity<?> changeUserEmail(@RequestBody EmailChangeRequest emailChangeRequest) {
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        int result = userService.updateEmailOfUsername(username, emailChangeRequest);

        return switch (result) {
            case 1 -> ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
            case 2 -> ResponseEntity.status(409).body(new SimpleStatusResponse("failed"));
            default -> ResponseEntity.ok(new SimpleStatusResponse("success"));
        };
    }

    @PostMapping("/account/change-password")
    public ResponseEntity<?> changePassword(@RequestBody PasswordChangeRequest passwordChangeRequest) {
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        int result = userService.updatePassword(username, passwordChangeRequest);

        return switch (result) {
            case 1 -> ResponseEntity.status(401).body(new SimpleStatusResponse("failed"));
            case 2 -> ResponseEntity.status(409).body(new SimpleStatusResponse("failed"));
            default -> ResponseEntity.ok(new SimpleStatusResponse("success"));
        };
    }

    @PostMapping("/account/delete-request")
    public ResponseEntity<?> deleteUser(@RequestBody AccountDeleteRequest accountDeleteRequest) {
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        int result = userService.deleteUserVerfied(username, accountDeleteRequest);

        return switch (result) {
            case 1 -> ResponseEntity.status(401).body(new SimpleStatusResponse("failed"));
            case 2 -> ResponseEntity.status(409).body(new SimpleStatusResponse("failed"));
            default -> ResponseEntity.ok(new SimpleStatusResponse("success"));
        };
    }
}
